// Skybox shader by Suso, for Mapwreck 3
// Based on https://www.shadertoy.com/view/WdX3D4

#version 150

uniform float GameTime;
uniform vec2 ScreenSize;
uniform mat4 ProjMat;
uniform mat4 ModelViewMat;

out vec4 fragColor;

float hash(float n) {
	return fract(sin(n) * 2711.01);
}

float pnoise(vec3 o) {
	vec3 p = floor(o); vec3 fr = fract(o);
		
	float n = p.x + p.y * 57.0 + p.z * 1009.0;

	float a = hash(n +    0.0);
	float b = hash(n +    1.0);
	float c = hash(n +   57.0);
	float d = hash(n +   58.0);
	float e = hash(n + 1009.0);
	float f = hash(n + 1010.0);
	float g = hash(n + 1066.0);
	float h = hash(n + 1067.0);
	
	vec3 t = fr * fr * (3.0 - 2.0 * fr);

	float res1 = a + (b - a) * t.x + (c - a) * t.y + (a - b + d - c) * t.x * t.y;
	float res2 = e + (f - e) * t.x + (g - e) * t.y + (e - f + h - g) * t.x * t.y;
	
	return mix(res1, res2, t.z);
}

float noise3d(vec3 p) {
    return (0.5 * pnoise(p) + 0.25 * pnoise(p * 3.0)) * 1.33;
}

float nebula(vec3 dir, float level, float power) {
    float nebula = pow(noise3d(2.0 * dir + 3.0), 20.0);
    if(nebula > 0.0) return pow(nebula * noise3d(dir.xyz * 10.0 * level + GameTime * 300.0) * 2.25, power);
	return 0.0;
}

float stars(vec3 dir) {
	return pow(pnoise(dir * 150.0 + GameTime * 100.0), 50.0);
}

void main() {
    vec4 cast_pos = normalize(inverse(ProjMat) * vec4((gl_FragCoord.xy / ScreenSize - 0.5) * 2.0, 1.0, 1.0));

    vec3 v = normalize(cast_pos.xyz * mat3(ModelViewMat));
	vec3 w = v.xzy + vec3(GameTime * 100.0, 0.0, 0.0);

    vec3 color = vec3(stars(v));
	color += nebula(v, 2.0, 0.5) * 1.5 * vec3(0.1, 0.8, 1.0);
    color += nebula(w, 2.5, 0.2) * 0.3 * vec3(0.8, 0.8, 0.8);

	fragColor = vec4(color, 1.0);

    fragColor.r +=  v.x *  v.y / 10.0;
    fragColor.g +=  v.z * -v.x / 10.0;
    fragColor.b += -v.x *  v.y / 10.0;
}
